/*
 * Home Health Grouper
 * Developer for the Center for Medicare and Medicaid Services CMS
 * by 3M Health Information Systems  for CMS Home Health.
 *
 * All code is provided as is.
 */

package com.mmm.cms.homehealth;

import java.util.ArrayList;
import java.util.List;

/**
 * Holds the HIPPS threshold information in order to assign a value based on 
 * Episode timing and number of visits in order to 
 * 
 * @author 3M Health Information Systems for CMS Home Health
 */
public class HIPPSThreshold {
	final private String timingValue;
	final private int visitsUpperLimit;
	final private int groupingStep;
	final private int equation;
	final private List<SeverityLimitValue> severityLimitValues;

	public HIPPSThreshold(String timingValue, int visitsUpperLimit, int groupingStep, int equation) {
		this.timingValue = timingValue;
		this.visitsUpperLimit = visitsUpperLimit;
		this.groupingStep = groupingStep;
		this.equation = equation;
		this.severityLimitValues = new ArrayList<SeverityLimitValue>();
	}
	
	public void addSeverityLimit(int lowerLimit, int upperLimit, char hippsSeverityValue)  {
		this.severityLimitValues.add(new SeverityLimitValue(lowerLimit, upperLimit, hippsSeverityValue));
	}
	
	public List<SeverityLimitValue> getSeverityLimits() {
		return this.severityLimitValues;
	}

	public String getTimingValue() {
		return timingValue;
	}

	public int getVisitsUpperLimit() {
		return visitsUpperLimit;
	}

	public int getGroupingStep() {
		return groupingStep;
	}

	public int getEquation() {
		return equation;
	}
	
	/**
	 * Returns the equation character representation based on the equation value
	 * 
	 * @param equationValue
	 * @return 
	 * @throws IllegalArgumentException if the equation value is out of range
	 */
	public char getHippsSeverityValue(int equationValue) {
		for (SeverityLimitValue limitValue : severityLimitValues)  {
			if (equationValue >= limitValue.getLowerLimit() && equationValue <= limitValue.getUpperLimit()) {
				return limitValue.getHippsSeverityValue();
			}
		}
		throw new IllegalArgumentException("Equation Value is not represented by this HIPPThreshold.  Equation value it out of range: " + equationValue);
	}
	
	public class SeverityLimitValue {
		
		final private int lowerLimit;
		final private int upperLimit;
		final private char hippsSeverityValue;

		public SeverityLimitValue(int lowerLimit, int upperLimit, char hippsSeverityValue) {
			if (lowerLimit > upperLimit) {
					throw new IllegalArgumentException("Low limit of " + lowerLimit + 
							" can not be larger than upperLimit of " + upperLimit);
			} else if (lowerLimit < 0) {
					throw new IllegalArgumentException("Low limit of " + lowerLimit + 
							" can not be less than 0");
			} else if (upperLimit < 0) {
					throw new IllegalArgumentException("Low limit of " + lowerLimit + 
							" can not be less than 0");
			} 

			this.lowerLimit = lowerLimit;
			this.upperLimit = upperLimit;
			this.hippsSeverityValue = hippsSeverityValue;
		}

		public int getLowerLimit() {
			return lowerLimit;
		}

		public int getUpperLimit() {
			return upperLimit;
		}

		public char getHippsSeverityValue() {
			return hippsSeverityValue;
		}

	}
}
