/*
 * Home Health Grouper
 * Developer for the Center for Medicare and Medicaid Services CMS
 * by 3M Health Information Systems  for CMS Home Health.
 *
 * All code is provided as is.
 */
package com.mmm.cms.homehealth.cpp;

import com.mmm.cms.homehealth.HomeHealthGrouperFactory;
import com.mmm.cms.homehealth.io.OasisReaderFactory;
import com.mmm.cms.homehealth.io.OasisRecordConverterIF;
import com.mmm.cms.homehealth.proto.CommonMessageText;
import com.mmm.cms.homehealth.proto.HomeHealthGrouperIF;
import com.mmm.cms.homehealth.proto.HomeHealthRecordIF;
import com.mmm.cms.homehealth.proto.ScoringResultsIF;
import com.mmm.cms.util.ScoringResultsFormatter;
import java.text.ParseException;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This class is used allow a DLL to connect to the Java Home Health Grouper
 * implementation. It initializes a Grouper Factory, then allows the record
 * scoring,. with the results formatted as previous groupers had done.
 *
 * @author 3M Health Information Systems for CMS Home Health
 */
public class DllToJavaConnector {

	public static boolean initComplete;
	/**
	 * set the interval default to 2 minutes
	 * This item can be controlled within the properties file with gcInterval=120000
	 * with the value being in milliseconds
	 */
	private static long gcInterval = 120000;
	private static long lastGC;	
	protected static HomeHealthGrouperFactory homeHealthFactory;
	protected static OasisReaderFactory readerFactory;

	/**
	 * This initializes the Java Grouper version for use within a CPP-DLL. It
	 * use environment variables in order to configure the grouper.
	 */
	public static void init() {

		if (!initComplete) {
			Logger.getLogger(DllToJavaConnector.class.getName()).log(Level.FINE, "HH-PPS: Initial Runtime memory available: {0}", Runtime.getRuntime().freeMemory());

			try {
				homeHealthFactory = new HomeHealthGrouperFactory();
				readerFactory = new OasisReaderFactory();

				// let the factory grab its own default properties file
				homeHealthFactory.init(null);
				initComplete = true;

				final Properties properties = homeHealthFactory.getProperties();
				if (properties != null) {
					final String strGcInterval = properties.getProperty("gcInterval");
					if (strGcInterval != null) {
						try {
							gcInterval = Long.parseLong(strGcInterval);
						} catch (NumberFormatException ex) {
							// don't care
						}
					}
				}
			} catch (Exception ex) {
				Logger.getLogger(DllToJavaConnector.class.getName()).log(Level.SEVERE, CommonMessageText.EVENT_MSG_HEADER, ex);
			}
		}
	}

	/**
	 * T
	 *
	 * @param record the Oasis B record as a string which gets converted to the
	 * Home Health record
	 * @return String in the format of HIPPS, OASIS TREATMENT, VERSION, FLAG as
	 * on continuous array of characters 5 for HIPPS, 18 for OASIS, 6 for
	 * version and 1 for flag.
	 */
	public static String scoreOasisB(String record) {
		String retCodes;
//        StringBuilder buffer;
		HomeHealthRecordIF homeHealthRecord;
		ScoringResultsIF scoreResult;
		HomeHealthGrouperIF grouper;

		Logger.getLogger(DllToJavaConnector.class.getName()).log(Level.INFO, 
				"HH-PPS: scoreOasisB() - record = ''{0}''", record);
		try {
			// initialize the connection to the home health grouper
			init();

			if (initComplete) {
				if (System.currentTimeMillis() - lastGC > gcInterval) {
					System.gc();
					lastGC = System.currentTimeMillis();
				}

				// convert the incoming record and score it
				homeHealthRecord = convertRecord(record);
				grouper = homeHealthFactory.getGrouper(homeHealthRecord);
				if (grouper != null) {
					// Grouper found, so score the record
					scoreResult = grouper.score(homeHealthRecord, false);

					retCodes = ScoringResultsFormatter.formatContinousScore(scoreResult);

					// clean up a little
					scoreResult.setClinicalValidator(null);

				} else {
					Logger.getLogger(DllToJavaConnector.class.getName()).log(Level.INFO, "HH-PPS: No Grouper version available to score the record: \"{0}\"", homeHealthRecord.toString());
					retCodes = ScoringResultsIF.DEFAULT_BLANK_VALUE;
				}
			} else {
				retCodes = ScoringResultsIF.DEFAULT_BLANK_VALUE;
			}

		} catch (Exception e) {
			Logger.getLogger(DllToJavaConnector.class.getName()).log(Level.SEVERE, "HH-PPS: Exception: "
					+ e.toString(), e);
			retCodes = ScoringResultsIF.DEFAULT_BLANK_VALUE;
		} catch (Error e) {
			Logger.getLogger(DllToJavaConnector.class.getName()).log(Level.SEVERE, "HH-PPS: Exception: "
					+ e.toString(), e);
			retCodes = ScoringResultsIF.DEFAULT_BLANK_VALUE;
		}

		return retCodes;
	}

	/**
	 * returns a HomeHealth record unless end of file, or an exception in
	 * reading the file or parsing the record.
	 *
	 * @param strRecord
	 * @return
	 */
	public static HomeHealthRecordIF convertRecord(String strRecord) {
		HomeHealthRecordIF record = null;
		OasisRecordConverterIF recordConverter;

		recordConverter = readerFactory.getRecordConverter(strRecord);
		if (recordConverter != null) {
			try {
				record = recordConverter.convertToHomeHealthRec(strRecord, 1, true);
			} catch (ParseException ex) {
				Logger.getLogger(DllToJavaConnector.class.getName()).log(Level.SEVERE, "HH-PPS: Could not convert record with cnoverter: " + recordConverter.getClass().getName(), ex);
			}
		}

		return record;
	}
	
	/**
	 * This provides the ability to send in a reusable record 'buffer' and have
	 * the results put into a reusable return 'buffer'. The return buffer must
	 * be 30 characters long.
	 * 
	 * @param record
	 * @param returnData 
	 */
	public static void scoreOasis(char[] record, char[] returnData) {
		final String strRecord = new String(record);
		String strReturn;
		int len;
		
		strReturn = scoreOasisB(strRecord);
		len = strReturn.length();
		for (int idx = 0; idx < len; idx++) {
			returnData[idx] = strReturn.charAt(idx);
		}
		
	}
	
}
