/*
 * Home Health Grouper
 * Developer for the Center for Medicare and Medicaid Services CMS
 * by 3M Health Information Systems  for CMS Home Health.
 *
 * All code is provided as is.
 */
package com.mmm.cms.homehealth.v4115;

import com.gdit.cms.hh.vut.CollectionValidationEditsIF;
import com.gdit.cms.hh.vut.OasisEditsEN;
import com.mmm.cms.homehealth.DataValidityFlag;
import com.mmm.cms.homehealth.proto.DataValidityFlagIF;
import com.mmm.cms.homehealth.proto.DiagnosisCodeIF;
import com.mmm.cms.homehealth.proto.HH_PPS_OasisC1EditsEN;
import com.mmm.cms.homehealth.proto.HomeHealthEventListenerIF;
import com.mmm.cms.homehealth.proto.HomeHealthGrouperIF;
import com.mmm.cms.homehealth.proto.HomeHealthRecordValidatorIF;
import com.mmm.cms.homehealth.proto.HomeHealthRecord_C1_IF;
import com.mmm.cms.homehealth.proto.HomeHealthRecordIF;
import static com.mmm.cms.homehealth.v3210.AbstractBaseValidator_v3210.ASSESSMENT_REASON_VALUES_ALL;
import com.mmm.cms.homehealth.v3312.ManifestationUtils;
import com.mmm.cms.homehealth.vut.CollectionValidationEdits;
import com.mmm.cms.homehealth.vut.HHOasisDataItem;
import com.mmm.cms.homehealth.vut.OasisValidationEdit;
import com.mmm.cms.util.IntegerUtils;
import com.mmm.cms.util.ScoringEventFormatter;
import com.mmm.cms.util.ValidateUtils;
import static com.mmm.cms.util.ValidateUtils.ARRAY_DOUBLE_ZERO_ONE_TWO;
import static com.mmm.cms.util.ValidateUtils.ARRAY_DOUBLE_ZERO_ONE_TWO_THREE;
import static com.mmm.cms.util.ValidateUtils.ARRAY_DOUBLE_ZERO_ONE_TWO_THREE_FOUR;
import static com.mmm.cms.util.ValidateUtils.ARRAY_DOUBLE_ZERO_ONE_TWO_THREE_FOUR_FIVE;
import static com.mmm.cms.util.ValidateUtils.ARRAY_DOUBLE_ZERO_ONE_TWO_THREE_FOUR_FIVE_SIX;
import static com.mmm.cms.util.ValidateUtils.NOT_APPLICABLE;
import static com.mmm.cms.util.ValidateUtils.ARRAY_NOT_APPLICABLE;
import static com.mmm.cms.util.ValidateUtils.ARRAY_UNKNOWN_UK;
import static com.mmm.cms.util.ValidateUtils.ARRAY_ZERO_ONE;
import java.util.Collection;
import java.util.logging.Logger;

/**
 * This Validator is different from the previous due to the OASIS-C1 record
 * specs defining the edit/error numbers and the use of the VUT for the actual
 * validation. For historical reasons, the similar methods are used for "valid"
 * indicator, however, the "is...INVLD" methods (which have been deprecated)
 * have been disabled in favor of the "is...Valid" methods.
 *
 * In order to determine valid items, this will check the
 * CollectionValidationEditsIF items to determine the error id and the
 * associated OASIS-C1 item.
 *
 * @author 3M Health Information Systems for CMS Home Health
 */
public class HomeHealthRecordClinicalValidator_v4115 implements HomeHealthRecordValidatorIF {

    public final static String[] ASSESSMENT_1_3_ONLY = new String[]{"01", "03"};
    public final static String[] ASSESSMENT_1_3_4_5_ONLY = new String[]{"01", "03", "04", "05"};
    public final static String[] ASSESSMENT_1_3_4_5_9_ONLY = new String[]{"01", "03", "04", "05", "09"};
    public final static String[] ASSESSMENT_1_3_9_ONLY = new String[]{"01", "03", "09"};
    public final static String[] ASSESSMENT_4_5_9_ONLY = new String[]{"04", "05", "09"};

    private boolean INTERNAL_LOGIC_Valid = true;
    /**
     * the overall data validity information
     */
    protected DataValidityFlagIF dataValidityFlag;
    protected int[] diagnosisScoringStatus;
    /**
     * VUT compatible edit information
     */
    protected CollectionValidationEditsIF edits;
    protected transient HomeHealthGrouperIF grouper;

    private boolean isRFA_1_3;
    private boolean isRFA_1_3_4_5;
    private boolean isRFA_1_3_9;
    private boolean isRFA_1_3_4_5_9;
    private boolean isRFA_4_5_9;

    /**
     * Constructor with the required Grouper reference
     *
     * @param grouper
     */
    public HomeHealthRecordClinicalValidator_v4115(HomeHealthGrouperIF grouper) {
        this.grouper = grouper;
        dataValidityFlag = new DataValidityFlag();
        diagnosisScoringStatus = new int[18];
        edits = new CollectionValidationEdits();
    }

    /**
     * Constructor with the required Grouper reference
     *
     * @param grouper
     * @param edits - this validator uses the supplied edits instead of
     * re-validating
     *
     */
    public HomeHealthRecordClinicalValidator_v4115(HomeHealthGrouperIF grouper,
            CollectionValidationEditsIF edits) {
        this.grouper = grouper;
        this.dataValidityFlag = new DataValidityFlag();
        this.diagnosisScoringStatus = new int[18];
        this.edits = edits;
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isASSMT_REASON_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isINFO_COMPLETED_DT_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isINTERNAL_LOGIC_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isTHERAPIES_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isVISION_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isPAIN_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isLESION_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isNPRSULC1_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isNPRSULC2_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isNPRSULC3_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isNPRSULC4_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isUNOBS_PRSULC_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isSTGPRSUL_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isNBR_STASULC_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isUNOBS_STASULC_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isSTATSTASIS_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isSTATSURG_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isDYSPNEIC_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isUR_INCONT_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isBWLINCONT_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isOSTOMY_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isCUR_DRESS_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isCUR_BATHING_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isCUR_TOILETING_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isCUR_TRANSFER_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isCUR_AMBULATION_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isCUR_INJECT_MEDS_INVLD() {
        throw new UnsupportedOperationException();
    }

    /**
     * @return @deprecated
     */
    @Override
    public boolean isTHER_NEED_INVLD() {
        throw new UnsupportedOperationException();
    }

    //////////////////////////////////////////
    @Override
    public boolean isASSMT_REASON_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M0100_ASSMT_REASON);
    }

    /**
     * This is not really necessary since the record has already been converted,
     * therefore the date was valid.
     *
     * @return true is there are no edit items for INFO COMPLETE
     */
    @Override
    public boolean isINFO_COMPLETED_DT_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M0090_INFO_COMPLETED_DT);
    }

    @Override
    public boolean isTHERAPIES_Valid() {

        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1030_THH_IV_INFUSION)
                && !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1030_THH_PAR_NUTRITION)
                && !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1030_THH_ENT_NUTRITION)
                && !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1030_THH_NONE_ABOVE);
    }

    @Override
    public boolean isINTERNAL_LOGIC_Valid() {
        return this.INTERNAL_LOGIC_Valid;
    }

    /**
     * @return
     */
    @Override
    public boolean isVISION_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1200_VISION);
    }

    @Override
    public boolean isPAIN_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1242_PAIN_FREQ_ACTVTY_MVMT);
    }

    @Override
    public boolean isLESION_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1350_LESION_OPEN_WND);
    }

    @Override
    public boolean isNPRSULC1_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1322_NBR_PRSULC_STG1);
    }

    @Override
    public boolean isNPRSULC2_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG2);
    }

    @Override
    public boolean isNPRSULC3_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG3);
    }

    @Override
    public boolean isNPRSULC4_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG4);
    }

    @Override
    public boolean isUNOBS_PRSULC_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_CVRG)
                && !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_DRSG)
                && !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_DEEP_TISUE);
    }

    @Override
    public boolean isSTGPRSUL_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1324_STG_PRBLM_ULCER);
    }

    @Override
    public boolean isNBR_STASULC_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1332_NBR_STAS_ULCR);
    }

    @Override
    public boolean isUNOBS_STASULC_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1330_STAS_ULCR_PRSNT);
    }

    @Override
    public boolean isSTATSTASIS_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1334_STUS_PRBLM_STAS_ULCR);
    }

    @Override
    public boolean isSTATSURG_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1340_SRGCL_WND_PRSNT);
    }

    @Override
    public boolean isDYSPNEIC_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1400_WHEN_DYSPNEIC);
    }

    @Override
    public boolean isUR_INCONT_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1610_UR_INCONT)
                && !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1615_INCNTNT_TIMING);
    }

    @Override
    public boolean isBWLINCONT_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1620_BWL_INCONT);
    }

    @Override
    public boolean isOSTOMY_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1630_OSTOMY);
    }

    @Override
    public boolean isCUR_DRESS_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1810_CRNT_DRESS_UPPER)
                && !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1820_CRNT_DRESS_LOWER);
    }

    @Override
    public boolean isCUR_BATHING_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1830_CRNT_BATHG);
    }

    @Override
    public boolean isCUR_TOILETING_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1840_CRNT_TOILTG);
    }

    @Override
    public boolean isCUR_TRANSFER_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1850_CRNT_TRNSFRNG);
    }

    @Override
    public boolean isCUR_AMBULATION_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1860_CRNT_AMBLTN);
    }

    /**
     *
     * @return @deprecated
     */
    @Override
    public boolean isCUR_INJECT_MEDS_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M2030_CRNT_MGMT_INJCTN_MDCTN);
    }

    @Override
    public boolean isTHER_NEED_Valid() {
        return !this.edits.isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M2200_THER_NEED_NBR);
    }

    @Override
    public void addEventListener(HomeHealthEventListenerIF listener) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    /**
     * gets the DataValidityFlag information
     *
     * @return the data validity flag object - this will never be null
     */
    @Override
    public DataValidityFlagIF getDataValidityFlag() {
        return dataValidityFlag;
    }

    /**
     * sets the data validity flag object - should not be null
     *
     * @param dataValidityFlag
     */
    @Override
    public void setDataValidityFlag(DataValidityFlagIF dataValidityFlag) {
        if (dataValidityFlag == null) {
            throw new IllegalArgumentException("dataValidityFlag parameter can not be null");
        }
        this.dataValidityFlag = dataValidityFlag;
    }

    /**
     * gets the ClinicalDomainDataIssueFlag
     *
     * @return
     */
    @Override
    public final boolean isClinicalDomainDataIssueFlag() {
        return dataValidityFlag.isClinicalIssue();
    }

    /**
     * gets the FunctionalDomainDataIssueFlag
     *
     * @return
     */
    @Override
    public final boolean isFunctionalDomainDataIssueFlag() {
        return dataValidityFlag.isFunctionalIssue();
    }

    /**
     * gets the ManifestationSequenceIssueFlag
     *
     * @return
     */
    @Override
    public final boolean isManifestationSequenceIssueFlag() {
        return dataValidityFlag.isManifestationSequenceIssue();
    }

    /**
     * gets the ServiceDomainDataIssueFlag
     *
     * @return
     */
    @Override
    public final boolean isServiceDomainDataIssueFlag() {
        return dataValidityFlag.isServiceIssue();
    }

    /**
     * Get the value of diagnosisScoringStatus
     *
     * @return the value of diagnosisScoringStatus
     */
    @Override
    public int[] getDiagnosisScoringStatus() {
        return diagnosisScoringStatus;
    }

    public CollectionValidationEditsIF getEdits() {
        return edits;
    }

    /**
     * Set the value of diagnosisScoringStatus
     *
     * @param diagnosisScoringStatus new value of diagnosisScoringStatus
     */
    @Override
    public void setDiagnosisScoringStatus(int[] diagnosisScoringStatus) {
        this.diagnosisScoringStatus = diagnosisScoringStatus;
    }

    @Override
    public boolean validate(HomeHealthRecordIF record) {
        validate(record, null);
        return edits.isEmpty();
    }

    @Override
    public CollectionValidationEditsIF validate(HomeHealthRecordIF record,
            Collection<HomeHealthEventListenerIF> listeners) {
        DiagnosisCodeIF code;
        int idx;

        //----------------------------------------
        // Clinical DOMAIN VALIDATION
        //----------------------------------------
        if (!validateASSMT_REASON(record)) {
            dataValidityFlag.setClinicalIssue(true);
            ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Assessment Reason", record.getASSMT_REASON());
        }

        // do the RFA comparison once per type
        isRFA_1_3 = ValidateUtils.isValidValue(record.getASSMT_REASON(), ASSESSMENT_1_3_ONLY);
        isRFA_1_3_4_5 = ValidateUtils.isValidValue(record.getASSMT_REASON(), ASSESSMENT_1_3_4_5_ONLY);
        isRFA_1_3_9 = ValidateUtils.isValidValue(record.getASSMT_REASON(), ASSESSMENT_1_3_9_ONLY);
        isRFA_1_3_4_5_9 = ValidateUtils.isValidValue(record.getASSMT_REASON(), ASSESSMENT_1_3_4_5_9_ONLY);
        isRFA_4_5_9 = ValidateUtils.isValidValue(record.getASSMT_REASON(), ASSESSMENT_4_5_9_ONLY);

        if (!validateEpisodeTiming(record)) {
            dataValidityFlag.setClinicalIssue(true);
            ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Episode Timing", record.getEPISODE_TIMING());
        }

        /*
         * Diagnosis code validations - start
         */
        if (!validateExistingCodes(record, listeners)) {
            dataValidityFlag.setClinicalIssue(true);
        }

        if (!validatePRIMARY_DIAG_ICD(record)) {
            dataValidityFlag.setClinicalIssue(true);
            ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Primary Diagnosis", record.getPRIMARY_DIAG_ICD().getCode());
        }

        // Nov 19, 2009 update from Errata
        // Notes:
        // 2. The edit stating where an ICD is not blank all prior ICDs within
        //  this item must not be blank is designed to assist the vendors in
        //  designing their systems; however, the State System will eliminate
        //  any embedded blank ICDs by shifting the subsequent non-blank ICDs
        //  into the prior blank items. Secondary ICD codes will not be moved
        // to Primary ICD codes.
//		if (!validateOtherDiagnosisSeries(record)) {
//			dataValidityFlag.setClinicalIssue(true);
//			ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Other Diagnosis not in a series - possible blank codes interspersed", "<various>");
//		}
        if (!validateDiagnosisUnique(record)) {
            dataValidityFlag.setClinicalIssue(true);
            ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Other Diagnosis are not unique", "<various>");
        }

        /*
         * only valid for RFA 1, 3, 4, 5
         */
        if (!validateTHH(record)) {
            dataValidityFlag.setClinicalIssue(true);
            ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Therapies Received at Home",
                    record.getTHH_IV_INFUSION() + ", "
                    + record.getTHH_PAR_NUTRITION() + ", "
                    + record.getTHH_ENT_NUTRITION() + ", "
                    + record.getTHH_NONE_ABOVE());

        }

        if (!validateVISION(record)) {
            dataValidityFlag.setClinicalIssue(true);
            ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Vision", record.getVISION());
        }

        if (!validateFREQ_PAIN(record)) {
            dataValidityFlag.setClinicalIssue(true);
            ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Frequent Pain", record.getPAIN_FREQ_ACTVTY_MVMT());
        }

        if (!validateNPRSULC(record, listeners)) {
            dataValidityFlag.setClinicalIssue(true);
            ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Number of Pressure Ulcers (4 stages)",
                    record.getNBR_PRSULC_STG1() + ", "
                    + record.getNBR_PRSULC_STG2() + ", "
                    + record.getNBR_PRSULC_STG3() + ", "
                    + record.getNBR_PRSULC_STG4());
        }

        if (!validateSTGPRSUL(record, listeners)) {
            dataValidityFlag.setClinicalIssue(true);
            ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Stage of Most Problematic Pressure Ulcer", record.getSTG_PRBLM_ULCER());
        }

        if (!validateStasisUlcer(record)) {
            dataValidityFlag.setClinicalIssue(true);
            ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Number of Stasis Ulcers", record.getNBR_STAS_ULCR());
        }

        if (!validateSTATSURG(record)) {
            dataValidityFlag.setClinicalIssue(true);
            ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Status Surgical Wound", record.getSTUS_PRBLM_SRGCL_WND());
        }

        if (!validateWHEN_DYSPNEIC(record)) {
            dataValidityFlag.setClinicalIssue(true);
            ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "When Dyspneic", record.getWHEN_DYSPNEIC());
        }

        if (!validateUR_INCONT(record)) {
            dataValidityFlag.setClinicalIssue(true);
            ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Urinary Incontinence", record.getUR_INCONT());
        }

        if (!validateBWL_INCONT(record)) {
            dataValidityFlag.setClinicalIssue(true);
            ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Bowl Incontinence", record.getBWL_INCONT());
        }

        if (!validateBWL_INCONT_OSTOMY(record)) {
            dataValidityFlag.setClinicalIssue(true);
            ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Bowl Incontinence Ostomy", record.getOSTOMY());
        }

        if (!validateLESION_OPEN_WND(record)) {
            dataValidityFlag.setClinicalIssue(true);
            ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Lesion Open Wound", record.getLESION_OPEN_WND());
        }

        if (!validateCUR_INJECT_MEDS(record)) {
            dataValidityFlag.setClinicalIssue(true);
            if (getEdits().isEditPresent(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M2000_DRUG_RGMN_RVW)) {
                ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null,
                        "Current Injectable Meds & Drug Regiment Review", record.getCRNT_MGMT_INJCTN_MDCTN() + " & " + ((HomeHealthRecord_C1_IF) record).getDRUG_RGMN_RVW());
            } else {
                ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Current Injectable Meds", record.getCRNT_MGMT_INJCTN_MDCTN());
            }
        }

        //----------------------------------------
        // FUNCTIONAL DOMAIN VALIDATION
        //----------------------------------------
        // check for functional issues on the record
        if (!validateCUR_DRESS(record)) {
            dataValidityFlag.setFunctionalIssue(true);
            ScoringEventFormatter.fireFunctionalIssueEvent(listeners, grouper, null, "Current Dressing (Upper & Lower)", record.getCRNT_DRESS_UPPER()
                    + ", "
                    + record.getCRNT_DRESS_LOWER());
        }

        if (!validateCUR_BATHING(record)) {
            dataValidityFlag.setFunctionalIssue(true);
            ScoringEventFormatter.fireFunctionalIssueEvent(listeners, grouper, null, "Current Bathing", record.getCRNT_BATHG());
        }

        if (!validateCUR_TOILETING(record)) {
            dataValidityFlag.setFunctionalIssue(true);
            ScoringEventFormatter.fireFunctionalIssueEvent(listeners, grouper, null, "Current Toileting", record.getCRNT_TOILTG());
        }

        if (!validateCUR_TRANSFERRING(record)) {
            dataValidityFlag.setFunctionalIssue(true);
            ScoringEventFormatter.fireFunctionalIssueEvent(listeners, grouper, null, "Current Transferring", record.getCRNT_TRNSFRNG());
        }

        if (!validateCUR_AMBULATION(record)) {
            dataValidityFlag.setFunctionalIssue(true);
            ScoringEventFormatter.fireFunctionalIssueEvent(listeners, grouper, null, "Current Ambulation", record.getCRNT_AMBLTN());
        }

        //--------------------------------------------------
        // SERVICE DOMAIN VALIDATION
        //--------------------------------------------------
        // check for service issues on the record
        if (!validateTHER_NEED(record)) {
            dataValidityFlag.setServiceIssue(true);
            ScoringEventFormatter.fireServiceIssueEvent(listeners, grouper, null, "Therapy Need and Number of Therapies",
                    record.getTHER_NEED_NBR() + ", "
                    + record.getTHER_NEED_NA());
        }

        validateDiagnosisCodes(record, listeners);

        //---------------------------------------------------
        // MANIFESTATION DOMAIN VALIDATION
        //---------------------------------------------------
        // check for manifestation issues on the record		
        if (this.edits.isEditPresent(HH_PPS_OasisC1EditsEN.EDIT_70020)
                || this.edits.isEditPresent(HH_PPS_OasisC1EditsEN.EDIT_70030)) {
            dataValidityFlag.setManifestationSequenceIssue(true);
        }

        //---------------------------------------------------
        // set the scoring information from record
        //---------------------------------------------------
        for (idx = 0; idx < 18; idx++) {
            code = record.getDiagnosisCode(idx);
            if (code.isValidForScoring()) {
                diagnosisScoringStatus[idx] = 2;
            } else if (code.isValidCode() || code.isOptionalPaymentCode()) {
                diagnosisScoringStatus[idx] = 1;
            } else {
                diagnosisScoringStatus[idx] = 0;
            }
        }

        return edits;
    }

    /**
     * Ensure a valid set of codes, and determines which codes to use in
     * scoring.
     *
     * November 2012 - added validation check for Optional Payment code pairing
     * checks
     *
     * @param record
     * @return true if the PDX is an allowable Principal code
     */
    public boolean validateDiagnosisCodes(HomeHealthRecordIF record, Collection<HomeHealthEventListenerIF> listeners) {
        boolean valid = true;
        int idx;
        DiagnosisCodeIF code;
        DiagnosisCodeIF tmpCode;
        boolean etiologyPairFound;

        if (isRFA_1_3_4_5) {

            /*
             * Due to the Primary awarding and the paired association of payment
             * type codes, column 3 and 4 are no longer relavent and so there is
             * no need to provide "edit" information, just the historic events
             */
            //------------------------------
            // Part 1: Determine if the code is valid or an
            //   optional payment code in order to determine which codes
            //   to use for scoring
            //------------------------------
            for (idx = 0; idx < 6; idx++) {
                code = record.getDiagnosisCode(idx);
                if (code.isValidForScoring()) {
                    // set the options codes as invalid for scoring
                    tmpCode = record.getOptionalDiagnosisCode3(idx);
                    if (!tmpCode.isEmpty()) {
                        tmpCode.setValidForScoring(false);
                        ScoringEventFormatter.fireIssueEvent(listeners, grouper, null,
                                "Diagnosis Code '" + tmpCode.getCode() + "' at position "
                                + (idx + 7) + " is NOT valid for scoring because code "
                                + code.getCode() + " in position "
                                + (idx + 1) + " is valid for scoring.");
                    }

                    tmpCode = record.getOptionalDiagnosisCode4(idx);
                    if (!tmpCode.isEmpty()) {
                        tmpCode.setValidForScoring(false);
                        ScoringEventFormatter.fireIssueEvent(listeners, grouper, null,
                                "Diagnosis Code '" + tmpCode.getCode() + "' at position "
                                + (idx + 13) + " is NOT valid for scoring because code "
                                + code.getCode() + "  in position "
                                + (idx + 1) + " is valid for scoring.");
                    }
                } else {
                    // check for being an optional payment code
                    if (code.isOptionalPaymentCode()) {
                        tmpCode = record.getOptionalDiagnosisCode4(idx);
                        if (tmpCode.isValidCode() && !tmpCode.isSecondaryOnly()) {
                            // the optional column 4 code is not valid
                            // so skip it - this is the default indicator on any
                            // invalid codes, so nothting to really do
                            tmpCode.setValidForScoring(false);
                            ScoringEventFormatter.fireIssueEvent(listeners, grouper, null,
                                    "Diagnosis Code '" + tmpCode.getCode() + "' at position "
                                    + (idx + 13) + " is NOT valid for scoring because it is NOT a manifestation code");
                        }

                        /*
                         * In version 3413, optional payment codes are paired with  
                         * a specific set of Dx codes.  If the pairing is not valid
                         * then the column 3 code can not score.  Blank codes are
                         * not checked.
                         */
                        tmpCode = record.getOptionalDiagnosisCode3(idx);
                        if (!tmpCode.isEmpty() && !code.isValidEtiologyPairing(tmpCode)) {
                            tmpCode.setValidForScoring(false);
                            ScoringEventFormatter.fireIssueEvent(listeners, grouper, null,
                                    "Diagnosis Code '" + tmpCode.getCode() + "' at position "
                                    + (idx + 7) + " is NOT valid for scoring because it is NOT a valid pair with code "
                                    + code.getCode() + " in position " + (idx + 1));
                        }
                    } else {
                        // set the payment codes to not be scorable
                        tmpCode = record.getOptionalDiagnosisCode3(idx);
                        if (!tmpCode.isEmpty()) {
                            tmpCode.setValidForScoring(false);
                            ScoringEventFormatter.fireIssueEvent(listeners, grouper, null,
                                    "Diagnosis Code '" + tmpCode.getCode() + "' at position "
                                    + (idx + 7) + " is NOT valid for scoring because code in position "
                                    + (idx + 1) + " is NOT an optional payment code.");
                        }

                        tmpCode = record.getOptionalDiagnosisCode4(idx);
                        if (!tmpCode.isEmpty()) {
                            tmpCode.setValidForScoring(false);
                            ScoringEventFormatter.fireIssueEvent(listeners, grouper, null,
                                    "Diagnosis Code '" + tmpCode.getCode() + "' at position "
                                    + (idx + 13) + " is NOT valid for scoring because code in position "
                                    + (idx + 1) + " is NOT an optional payment code.");
                        }
                    }
                }
            }

            //------------------------------
            // Part 2: Flag acceptable pairing of manifestion diagnosis
            //   with etiologies; disqualify others
            //------------------------------
            for (idx = 1; idx < 6; idx++) {
                code = record.getDiagnosisCode(idx);

                /*
                 * Since there are now two types of manifestations: HH and MCE,
                 * only check those codes that are secondary and have a Diagnostic 
                 * Group with an id no which means that they are part of HH
                 */
                if (code.isSecondaryOnly() && code.getDiagnosticGroup().getId() > 0) {
                    // set the valid for scoring based on whether there
                    // is an exceptable pairing code found for this code
                    etiologyPairFound = ManifestationUtils.getPairedCode(grouper, record, code, idx) != null;
                    code.setValidForScoring(etiologyPairFound);

                    // Oct 2012 - added to ensure flag was identfied
                    // set the manifestion flag
                    if (!etiologyPairFound) {
                        this.edits.add(new OasisValidationEdit(HH_PPS_OasisC1EditsEN.EDIT_70030,
                                new HHOasisDataItem("M1023_OTH_DIAG" + idx + "_ICD", code.getCode())));
                        ScoringEventFormatter.fireIssueEvent(listeners, grouper, null,
                                "Diagnosis Code '" + code.getCode() + "' at position "
                                + (idx + 1) + " - Edit: " + HH_PPS_OasisC1EditsEN.EDIT_70030.getDescription());

                        valid = false;
                    }
                }
            }

            //------------------------------
            // Part 3: Flag acceptable pairing of manifestion diagnosis
            //   column 4 with etiologies; disqualify others
            //------------------------------
            for (idx = 0; idx < 6; idx++) {
                code = record.getOptionalDiagnosisCode4(idx);

                // only check codes that are still valid to score
                if (code.isValidForScoring()
                        && code.isSecondaryOnly()) {
                    tmpCode = record.getOptionalDiagnosisCode3(idx);
                    etiologyPairFound = code.isValidEtiologyPairing(tmpCode);
                    code.setValidForScoring(etiologyPairFound);
                    if (!etiologyPairFound) {
                        this.edits.add(new OasisValidationEdit(HH_PPS_OasisC1EditsEN.EDIT_70030,
                                new HHOasisDataItem("M1025_PMT_DIAG_ICD_" + ('A' + idx) + "4", code.getCode())));
                        ScoringEventFormatter.fireIssueEvent(listeners, grouper, null,
                                "Diagnosis Code '" + code.getCode() + "' at position "
                                + (idx + 7) + " - Edit: " + HH_PPS_OasisC1EditsEN.EDIT_70030.getDescription());

                        valid = false;
                    }
                }
            }
        }

        return valid;
    }

    /**
     * Pseudo code lines: no corresponding line
     *
     * @param record
     * @return
     */
    public boolean validateASSMT_REASON(HomeHealthRecordIF record) {
        boolean bool = true;

        if (!ValidateUtils.isValidValue(record.getASSMT_REASON(), ASSESSMENT_REASON_VALUES_ALL)) {
            this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060, new HHOasisDataItem("M0100_ASSMT_REASON", record.getASSMT_REASON())));
            bool = false;
        }
        return bool;
    }

    public boolean validateEpisodeTiming(HomeHealthRecordIF record) {
        boolean bool = true;

        if (!ValidateUtils.isValidValue(record.getEPISODE_TIMING(), ValidateUtils.ARRAY_DOUBLE_ONE_TWO, ValidateUtils.ARRAY_UNKNOWN_UK, ValidateUtils.ARRAY_NOT_APPLICABLE)) {
            this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060, new HHOasisDataItem("M0110_EPISODE_TIMING", record.getEPISODE_TIMING())));
            bool = false;
        }
        return bool;
    }

    /**
     * Ensures that all non-blank codes are valid Codes within the record. Note
     * that if all the codes are blank, this method will return true, i.e.
     * valid. This does not indicate which diagnosis is invalid.
     *
     * Added as part of the increased specificity validation of the record
     * pertaining to the ICD-9-Codes
     *
     * @param record
     * @return true if all the codes are blank and/or valid Codes for the
     * current HH-PPS version
     */
    public boolean validateExistingCodes(HomeHealthRecordIF record,
            Collection<HomeHealthEventListenerIF> listeners) {
        DiagnosisCodeIF code;
        boolean valid = true;

        if (isRFA_1_3_4_5) {
            for (int idx = 0; idx < 6; idx++) {
                code = record.getDiagnosisCode(idx);

                // Blank codes are acceptable, but invalid codes
                // are not.
                if (!code.isValidCode() && !code.isEmpty()) {
                    ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Invalid Diagnosis code", code.getCode());

                    this.edits.add(new OasisValidationEdit(HH_PPS_OasisC1EditsEN.EDIT_70000,
                            new HHOasisDataItem(idx == 0 ? HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1021_PRIMARY_DIAG_ICD : "M1023_OTH_DIAG" + (idx + 1) + "_ICD", code.getCode())));

                    valid = false;
                }
            }

            for (int idx = 0; idx < 6; idx++) {
                code = record.getOptionalDiagnosisCode3(idx);
                // Blank codes are acceptable, but invalid codes
                // are not.
                if (!code.isValidCode() && !code.isEmpty()) {
                    ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Invalid Diagnosis code", code.getCode());
                    this.edits.add(new OasisValidationEdit(HH_PPS_OasisC1EditsEN.EDIT_70000,
                            new HHOasisDataItem("M1025_PMT_DIAG_ICD_" + ('A' + idx) + "3", code.getCode())));
                    valid = false;
                }

                code = record.getOptionalDiagnosisCode4(idx);
                // Blank codes are acceptable, but invalid codes
                // are not.
                if (!code.isValidCode() && !code.isEmpty()) {
                    ScoringEventFormatter.fireClinicalIssueEvent(listeners, grouper, null, "Invalid Diagnosis code", code.getCode());
                    this.edits.add(new OasisValidationEdit(HH_PPS_OasisC1EditsEN.EDIT_70000,
                            new HHOasisDataItem("M1025_PMT_DIAG_ICD_" + ('A' + idx) + "4", code.getCode())));
                    valid = false;
                }
            }
        }

        return valid;
    }

//	/**
//	 * From OASIS-C spec: *2. If M1022_OTH_DIAG1_ICD is blank, then
//	 * M1022_OTH_DIAG2_ICD must be blank.
//	 *
//	 * Similar validation above is repeated for diagnosis codes through
//	 * OTH_DIAG5_ICD.
//	 *
//	 * Once a blank is encountered, the rest of the codes in this column, if any
//	 * are marked as not valid for scoring
//	 *
//	 * @param record
//	 * @return
//	 */
//	public boolean validateOtherDiagnosisSeries(HomeHealthRecordIF record) {
//		DiagnosisCodeIF previousCode;
//		DiagnosisCodeIF currentCode;
//		boolean valid = true;
//
//		for (int idx = 1; idx < 5;) {
//			previousCode = record.getDiagnosisCode(idx++);
//			currentCode = record.getDiagnosisCode(idx);
//
//			// if the previous code is blank and the current code it not
//			// then the codes were not enterred serially
//			if (previousCode.isEmpty() && !currentCode.isEmpty()) {
//				this.edits.add(new OasisValidationEdit(HH_PPS_OasisC1EditsEN.EDIT_70050,
//						new HHOasisDataItem("M1023_OTH_DIAG" + idx + "_ICD", "")));
//
//				// mark the current and the rest of the codes
//				// in this first column as unscorable because of the
//				// preceding blank
//				currentCode.setValidForScoring(false);
//				while (++idx < 6) {
//					currentCode = record.getDiagnosisCode(idx);
//					currentCode.setValidForScoring(false);
//				}
//				valid = false;
//				break;
//			}
//		}
//		return valid;
//	}
//
    /**
     * From OASIS-C spec: *1. If M1022_OTH_DIAG2_ICD is not blank, it cannot be
     * equal to M1020_PRIMARY_DIAG_ICD, M1022_OTH_DIAG1_ICD,
     * M1022_OTH_DIAG3_ICD, M1022_OTH_DIAG4_ICD, or M1022_OTH_DIAG5_ICD.
     *
     * Similar validation above is repeated for diagnosis codes through
     * OTH_DIAG5_ICD.
     *
     * The codes are checked from top to bottom. If a code is a duplicated in
     * subsequent positions, then the subsequent codes are marked as not valid
     * for scoring. However, the first code will still be able to score.
     *
     * OASIS-C1 - edit -5000 - no duplicate DX
     *
     * @param record
     * @return
     */
    public boolean validateDiagnosisUnique(HomeHealthRecordIF record) {
        DiagnosisCodeIF otherCode;
        DiagnosisCodeIF currentCode;
        boolean valid = true;

        if (isRFA_1_3_4_5) {
            // check only code 1 through 5 in this outter loop
            // the inner loop will check against the 6th code
            for (int idx = 0; idx < 5 && valid;) {
                currentCode = record.getDiagnosisCode(idx++);

                // only check valid codes
                if (currentCode.isValidCode()) {
                    for (int j = idx; j < 6;) {
                        // if the current code is duplicated in subsequent positions
                        // then mark the subsequent positions as not valid for
                        // scoring, but allow the current code's to scoring
                        // indicator to remain
                        otherCode = record.getDiagnosisCode(j++);
                        if (otherCode.isValidCode()
                                && currentCode.equals(otherCode)) {
                            // set the code to not score
                            otherCode.setValidForScoring(false);

                            this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5000,
                                    new HHOasisDataItem("M1023_OTH_DIAG" + j + "_ICD", otherCode.getCode())));

                            // indicate that a duplicate was found
                            valid = false;
                            break;
                        }
                    }
                }
            }
        }

        return valid;
    }

    /**
     * If the super.validatePRIMARY_DIAG_ICD() is false, then check for the
     * primary being blank. If the primary is blank, then set all the rest of
     * the codes to not score.
     *
     * @param record
     * @return
     */
    public boolean validatePRIMARY_DIAG_ICD(HomeHealthRecordIF record) {
        boolean valid = true;
        DiagnosisCodeIF code = record.getPRIMARY_DIAG_ICD();

        if (isRFA_1_3_4_5) {
            // check for the primary being blank
            if (code.isEmpty()) {
                // set all the other codes in the record to not score
                for (int idx = 1; idx < 18;) {
                    code = record.getDiagnosisCode(idx++);
                    if (code != null) {
                        code.setValidForScoring(false);
                    }
                }

                this.edits.add(new OasisValidationEdit(HH_PPS_OasisC1EditsEN.EDIT_70000,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1021_PRIMARY_DIAG_ICD, code.getCode())));
                this.edits.add(new OasisValidationEdit(HH_PPS_OasisC1EditsEN.EDIT_70010,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1021_PRIMARY_DIAG_ICD, code.getCode())));
                valid = false;

            } else if (!code.isValidCode()) {
                this.edits.add(new OasisValidationEdit(HH_PPS_OasisC1EditsEN.EDIT_70000,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1021_PRIMARY_DIAG_ICD, code.getCode())));
                valid = false;
            } else {
                if (code.isSecondaryOnly()) {
                    /*
                     * Secondary/Manifestation codes can not be in the primary position
                     */
                    this.edits.add(new OasisValidationEdit(HH_PPS_OasisC1EditsEN.EDIT_70020,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1021_PRIMARY_DIAG_ICD, code.getCode())));
                    code.setValidForScoring(false);
                    valid = false;
                } else if (code.isExternalCauseCode()) {
                    /*
                     * check for E-code (External cause code) validation
                     */
                    this.edits.add(new OasisValidationEdit(HH_PPS_OasisC1EditsEN.EDIT_70010,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1021_PRIMARY_DIAG_ICD, code.getCode())));
                    code.setValidForScoring(false);
                    valid = false;
                }
            }
        }

        return valid;
    }

    /**
     * Oasis-C 1030_THH values. Therapies Received at Home
     *
     * If the Therapies are invalid, it sets the Internal_Logic_Invld = true
     * Therapies_Invld = true
     *
     * Only checked for Assessment Reason 01, 03, 04, 05
     *
     * param record return true if there are no errors
     */
    public boolean validateTHH(HomeHealthRecordIF record) {
        boolean valid = true;

        if (isRFA_1_3_4_5) {
            if (!ValidateUtils.isValidValue(record.getTHH_IV_INFUSION(), ARRAY_ZERO_ONE)) {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1030_THH_IV_INFUSION, record.getTHH_IV_INFUSION())));
            }

            if (!ValidateUtils.isValidValue(record.getTHH_PAR_NUTRITION(), ARRAY_ZERO_ONE)) {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1030_THH_PAR_NUTRITION, record.getTHH_PAR_NUTRITION())));
            }

            if (!ValidateUtils.isValidValue(record.getTHH_ENT_NUTRITION(), ARRAY_ZERO_ONE)) {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1030_THH_ENT_NUTRITION, record.getTHH_ENT_NUTRITION())));
            }

            if (!ValidateUtils.isValidValue(record.getTHH_NONE_ABOVE(), ARRAY_ZERO_ONE)) {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1030_THH_NONE_ABOVE, record.getTHH_NONE_ABOVE())));
            }

            if ("1".equals(record.getTHH_NONE_ABOVE())
                    && ("1".equals(record.getTHH_IV_INFUSION())
                    || "1".equals(record.getTHH_PAR_NUTRITION())
                    || "1".equals(record.getTHH_ENT_NUTRITION()))) {
                this.INTERNAL_LOGIC_Valid = false;
            }

            if ("0".equals(record.getTHH_NONE_ABOVE())
                    && "0".equals(record.getTHH_IV_INFUSION())
                    && "0".equals(record.getTHH_PAR_NUTRITION())
                    && "0".equals(record.getTHH_ENT_NUTRITION())) {
                this.INTERNAL_LOGIC_Valid = false;
            }

            valid |= this.INTERNAL_LOGIC_Valid;
        }

        return valid;
    }

    /**
     * Oasis-C M1200_VISION Vision
     *
     * Only checked for Assessment Reason 01, 03, 04, 05
     *
     * @param record
     * @return
     */
    public boolean validateVISION(HomeHealthRecordIF record) {
        boolean valid = true;

        if (isRFA_1_3_4_5
                && !ValidateUtils.isValidValue(record.getVISION(), ARRAY_DOUBLE_ZERO_ONE_TWO)) {
            valid = false;
            this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                    new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1200_VISION, record.getVISION())));
        }
        return valid;
    }

    /**
     * Oasis-C M1242_PAIN_FREQ_ACTVTY_MVMT Frequent pain during activity or
     * movement
     *
     * Only checked for Assessment Reason 01, 03, 04, 05
     *
     * @param record
     * @return
     */
    public boolean validateFREQ_PAIN(HomeHealthRecordIF record) {
        boolean valid = true;

        if (isRFA_1_3_4_5_9
                && !ValidateUtils.isValidValue(record.getPAIN_FREQ_ACTVTY_MVMT(), ARRAY_DOUBLE_ZERO_ONE_TWO_THREE_FOUR)) {
            valid = false;
            this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                    new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1242_PAIN_FREQ_ACTVTY_MVMT, record.getPAIN_FREQ_ACTVTY_MVMT())));
        }
        return valid;
    }

    /**
     * Oasis-C M1350_LESION_OPEN_WND Lesion Open Wound
     *
     * Only checked for Assessment Reason 01, 03, 04, 05
     *
     * @param record
     * @return Ulcers
     */
    public boolean validateLESION_OPEN_WND(HomeHealthRecordIF record) {
        boolean valid = true;

        if (isRFA_1_3
                && !ValidateUtils.isValidValue(record.getLESION_OPEN_WND(), ARRAY_ZERO_ONE)) {
            valid = false;
            this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                    new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1350_LESION_OPEN_WND, record.getLESION_OPEN_WND())));
        }

        return valid;
    }

    /**
     * Logic updated Aug 31, 2009
     *
     * M1322_NBR_PRSULC_STG1 No. Pressure Ulcers -Stage 1
     *
     * M1308_NBR_PRSULC_STG2 No. Pressure Ulcers -Stage 2
     *
     * M1308_NBR_PRSULC_STG3 No. Pressure Ulcers -Stage 3
     *
     * M1308_NBR_PRSULC_STG4 No. Pressure Ulcers -Stage 4
     *
     * Only checked for Assessment Reason 01, 03, 04, 05 - Some associated
     * fields within this validation are only available during Assessment Reason
     * 1 & 3
     *
     * @param record
     * @return
     */
    public boolean validateNPRSULC(HomeHealthRecordIF record,
            Collection<HomeHealthEventListenerIF> listeners) {
        boolean valid = true;

        if (isRFA_1_3_4_5_9) {
            if (record instanceof HomeHealthRecord_C1_IF) {
                final HomeHealthRecord_C1_IF recordC = (HomeHealthRecord_C1_IF) record;
                final String ASSMT_REASON = recordC.getASSMT_REASON();
                final String UNHLD_STG2_PRSR_ULCR = recordC.getUNHLD_STG2_PRSR_ULCR();
                final String STG_PRBLM_ULCER = recordC.getSTG_PRBLM_ULCER();
                final String NBR_PRSULC_STG1 = recordC.getNBR_PRSULC_STG1();
                final String NBR_PRSULC_STG2 = recordC.getNBR_PRSULC_STG2();
                final String NBR_PRSULC_STG3 = recordC.getNBR_PRSULC_STG3();
                final String NBR_PRSULC_STG4 = recordC.getNBR_PRSULC_STG4();
                final String NSTG_CVRG = recordC.getNSTG_CVRG();
                final String NSTG_DEEP_TISUE = recordC.getNSTG_DEEP_TISUE();
                final String NSTG_DRSG = recordC.getNSTG_DRSG();
                final String STUS_PRBLM_PRSR_ULCR = recordC.getSTUS_PRBLM_PRSR_ULCR();

                if (!ValidateUtils.isEmpty(UNHLD_STG2_PRSR_ULCR) && !ValidateUtils.isNumeric(UNHLD_STG2_PRSR_ULCR)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1306_UNHLD_STG2_PRSR_ULCR, UNHLD_STG2_PRSR_ULCR)));
                }
                
                if (!ValidateUtils.isEmptyCaret(NBR_PRSULC_STG2) && !ValidateUtils.isNumeric(NBR_PRSULC_STG2)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG2, NBR_PRSULC_STG2)));
                }
                if (!ValidateUtils.isEmptyCaret(NBR_PRSULC_STG3) && !ValidateUtils.isNumeric(NBR_PRSULC_STG3)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG3, NBR_PRSULC_STG3)));
                }
                if (!ValidateUtils.isEmptyCaret(NBR_PRSULC_STG4) && !ValidateUtils.isNumeric(NBR_PRSULC_STG4)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG4, NBR_PRSULC_STG4)));
                }
                if (!ValidateUtils.isEmptyCaret(NSTG_CVRG) && !ValidateUtils.isNumeric(NSTG_CVRG)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_CVRG, NSTG_CVRG)));
                }
                if (!ValidateUtils.isEmptyCaret(NSTG_DEEP_TISUE) && !ValidateUtils.isNumeric(NSTG_DEEP_TISUE)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_DEEP_TISUE, NSTG_DEEP_TISUE)));
                }

                if (!ValidateUtils.isEmptyCaret(NSTG_DRSG) && !ValidateUtils.isNumeric(NSTG_DRSG)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_DRSG, NSTG_DRSG)));
                }
                if (isRFA_1_3_9
                        && !ValidateUtils.isValidValue(STUS_PRBLM_PRSR_ULCR, ValidateUtils.ARRAY_DOUBLE_ZERO_ONE_TWO_THREE, ValidateUtils.ARRAY_CARET_VALUES, new String[]{ValidateUtils.NOT_APPLICABLE})) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1320_STUS_PRBLM_PRSR_ULCR, STUS_PRBLM_PRSR_ULCR)));
                }

                if (!ValidateUtils.isValidValue(STG_PRBLM_ULCER, ValidateUtils.ARRAY_DOUBLE_ONE_TWO_THREE_FOUR, new String[]{ValidateUtils.NOT_APPLICABLE})) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1324_STG_PRBLM_ULCER, STG_PRBLM_ULCER)));
                }

                /*ARRAY_
                 * edit 5110
                 */
                if ("0".equals(UNHLD_STG2_PRSR_ULCR)) {
                    if (!ValidateUtils.isEmpty(NBR_PRSULC_STG2)
                            || !ValidateUtils.isEmpty(NBR_PRSULC_STG3)
                            || !ValidateUtils.isEmpty(NBR_PRSULC_STG4)
                            || !ValidateUtils.isEmpty(NSTG_CVRG)
                            || !ValidateUtils.isEmpty(NSTG_DEEP_TISUE)
                            || !ValidateUtils.isEmpty(NSTG_DRSG)
                            || !ValidateUtils.isEmpty(STUS_PRBLM_PRSR_ULCR)) {
                        valid = false;
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5110,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1306_UNHLD_STG2_PRSR_ULCR, UNHLD_STG2_PRSR_ULCR)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5110,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG2, NBR_PRSULC_STG2)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5110,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG3, NBR_PRSULC_STG3)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5110,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG4, NBR_PRSULC_STG4)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5110,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_CVRG, NSTG_CVRG)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5110,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_DRSG, NSTG_DRSG)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5110,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_DEEP_TISUE, NSTG_DEEP_TISUE)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5110,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1320_STUS_PRBLM_PRSR_ULCR, STUS_PRBLM_PRSR_ULCR)));
                    }
                } else if ("1".equals(UNHLD_STG2_PRSR_ULCR)) {
                    if (ValidateUtils.isEmpty(NBR_PRSULC_STG2)
                            || ValidateUtils.isEmpty(NBR_PRSULC_STG3)
                            || ValidateUtils.isEmpty(NBR_PRSULC_STG4)
                            || ValidateUtils.isEmpty(NSTG_CVRG)
                            || ValidateUtils.isEmpty(NSTG_DEEP_TISUE)
                            || ValidateUtils.isEmpty(NSTG_DRSG)) {
                        valid = false;
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5120,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1306_UNHLD_STG2_PRSR_ULCR, UNHLD_STG2_PRSR_ULCR)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5120,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG2, NBR_PRSULC_STG2)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5120,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG3, NBR_PRSULC_STG3)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5120,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG4, NBR_PRSULC_STG4)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5120,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_CVRG, NSTG_CVRG)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5120,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_DRSG, NSTG_DRSG)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5120,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_DEEP_TISUE, NSTG_DEEP_TISUE)));

                    } else if (ValidateUtils.DOUBLE_ZERO.equals(NBR_PRSULC_STG2)
                            && ValidateUtils.DOUBLE_ZERO.equals(NBR_PRSULC_STG3)
                            && ValidateUtils.DOUBLE_ZERO.equals(NBR_PRSULC_STG4)
                            && ValidateUtils.DOUBLE_ZERO.equals(NSTG_CVRG)
                            && ValidateUtils.DOUBLE_ZERO.equals(NSTG_DRSG)
                            && ValidateUtils.DOUBLE_ZERO.equals(NSTG_DEEP_TISUE)) {
                        valid = false;
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5130,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG2, NBR_PRSULC_STG2)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5130,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG3, NBR_PRSULC_STG3)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5130,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG4, NBR_PRSULC_STG4)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5130,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_CVRG, NSTG_CVRG)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5130,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_DRSG, NSTG_DRSG)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_5130,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_DEEP_TISUE, NSTG_DEEP_TISUE)));
                    }
                } else {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1306_UNHLD_STG2_PRSR_ULCR, UNHLD_STG2_PRSR_ULCR)));
                }

                /*
                 * edit 4210
                 */
                if (ValidateUtils.DOUBLE_ZERO.equals(NBR_PRSULC_STG1)
                        && (ValidateUtils.DOUBLE_ZERO.equals(NBR_PRSULC_STG2) || ValidateUtils.isEmpty(NBR_PRSULC_STG2))
                        && (ValidateUtils.DOUBLE_ZERO.equals(NBR_PRSULC_STG3) || ValidateUtils.isEmpty(NBR_PRSULC_STG3))
                        && (ValidateUtils.DOUBLE_ZERO.equals(NBR_PRSULC_STG4) || ValidateUtils.isEmpty(NBR_PRSULC_STG4))
                        && !ValidateUtils.NOT_APPLICABLE.equals(STG_PRBLM_ULCER)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4210,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1322_NBR_PRSULC_STG1, NBR_PRSULC_STG1)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4210,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG2, NBR_PRSULC_STG2)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4210,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG3, NBR_PRSULC_STG3)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4210,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG4, NBR_PRSULC_STG4)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4210,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1324_STG_PRBLM_ULCER, STG_PRBLM_ULCER)));
                }

                /*
                 * convert the strings to number to makei it easier to check 
                 * greater than, because the greater than check no longer works
                 * with a string since "^ " (i.e. empty) is actually greater
                 * than "00"
                 */
                final int intStage2 = ValidateUtils.isEmpty(NBR_PRSULC_STG2) ? -1 : IntegerUtils.parseInt(NBR_PRSULC_STG2, -1);
                final int intStage3 = ValidateUtils.isEmpty(NBR_PRSULC_STG3) ? -1 : IntegerUtils.parseInt(NBR_PRSULC_STG3, -1);
                final int intStage4 = ValidateUtils.isEmpty(NBR_PRSULC_STG4) ? -1 : IntegerUtils.parseInt(NBR_PRSULC_STG4, -1);
                final int intNonStageCovering = ValidateUtils.isEmpty(NSTG_CVRG) ? -1 : IntegerUtils.parseInt(NSTG_CVRG, -1);
                final int intNonStageDeepTissue = ValidateUtils.isEmpty(NSTG_DEEP_TISUE) ? -1 : IntegerUtils.parseInt(NSTG_DEEP_TISUE, -1);

                /*
                 * edit 4130
                 */
                if (intStage2 == 0 && intStage3 == 0 && intStage4 == 0 && intNonStageCovering == 0
                        && intNonStageDeepTissue == 0
                        && isRFA_1_3_9
                        && !ValidateUtils.NOT_APPLICABLE.equals(STUS_PRBLM_PRSR_ULCR)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4130,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG2, NBR_PRSULC_STG2)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4130,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG3, NBR_PRSULC_STG3)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4130,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG4, NBR_PRSULC_STG4)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4130,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_CVRG, NSTG_CVRG)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4130,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_DEEP_TISUE, NSTG_DEEP_TISUE)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4130,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1320_STUS_PRBLM_PRSR_ULCR, STUS_PRBLM_PRSR_ULCR)));
                }

                /*
                 * edit 4140
                 */
                if ((intStage2 > 0 || intNonStageDeepTissue > 0)
                        && intStage3 == 0 && intStage4 == 0 && intNonStageCovering == 0
                        && isRFA_1_3_9
                        && !"03".equals(STUS_PRBLM_PRSR_ULCR)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4140,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG2, NBR_PRSULC_STG2)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4140,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG3, NBR_PRSULC_STG3)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4140,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG4, NBR_PRSULC_STG4)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4140,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_CVRG, NSTG_CVRG)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4140,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_DEEP_TISUE, NSTG_DEEP_TISUE)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4140,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1320_STUS_PRBLM_PRSR_ULCR, STUS_PRBLM_PRSR_ULCR)));
                }

                /*
                 * edit 4150
                 */
                if ((intStage3 > 0 || intStage4 > 0)
                        && isRFA_1_3_9
                        && !ValidateUtils.isValidValue(STUS_PRBLM_PRSR_ULCR, ARRAY_DOUBLE_ZERO_ONE_TWO_THREE)) {
                    // if any stage x > 0, then the Status Problem Pressure Ucler
                    // must be 00, 01, 02, or 03, but only when RFA = 1 or 3
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4150,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG3, NBR_PRSULC_STG3)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4150,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG4, NBR_PRSULC_STG4)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4150,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1320_STUS_PRBLM_PRSR_ULCR, STUS_PRBLM_PRSR_ULCR)));
                }

                /*
                 * edit 4160
                 */
                if (intNonStageCovering > 0 && intStage2 == 0 && intStage3 == 0
                        && intStage4 == 0 && intNonStageDeepTissue == 0
                        && isRFA_1_3_9
                        && !("02".equals(STUS_PRBLM_PRSR_ULCR) || "03".equals(STUS_PRBLM_PRSR_ULCR))) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4160,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG2, NBR_PRSULC_STG2)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4160,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG3, NBR_PRSULC_STG3)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4160,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG4, NBR_PRSULC_STG4)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4160,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_CVRG, NSTG_CVRG)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4160,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NSTG_DEEP_TISUE, NSTG_DEEP_TISUE)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4160,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1320_STUS_PRBLM_PRSR_ULCR, STUS_PRBLM_PRSR_ULCR)));
                }

                /*
                 * edit 4170
                 * edit 3060 for stage 1
                 */
                if (ValidateUtils.isValidValue(NBR_PRSULC_STG1, ARRAY_DOUBLE_ZERO_ONE_TWO_THREE_FOUR)) {
                    if ("01".equals(STG_PRBLM_ULCER)
                            && !ValidateUtils.isValidValue(NBR_PRSULC_STG1, ValidateUtils.ARRAY_DOUBLE_ONE_TWO_THREE_FOUR)) {
                        valid = false;
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4170,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1322_NBR_PRSULC_STG1, NBR_PRSULC_STG1)));
                    }
                } else {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1322_NBR_PRSULC_STG1, NBR_PRSULC_STG1)));
                }
            }
        }

        return valid;
    }

    /**
     * Oasis-C M1324_STG_PRBLM_ULCER Stage of Most Problematic Pressure Ulcer
     *
     * Only checked for Assessment Reason 01, 03, 04, 05
     *
     * @param record
     * @return
     */
    public boolean validateSTGPRSUL(HomeHealthRecordIF record,
            Collection<HomeHealthEventListenerIF> listeners) {
        String tmpStr;
        boolean valid = true;

        if (isRFA_1_3_4_5_9) {
            final String problemUlcer = record.getSTG_PRBLM_ULCER();

            // check the value of the Stage of Most Problematic Pressure Ulcer
            if ("01".equals(problemUlcer)) {
                // number of pressure Ulcer stage 1 must be > 0
                tmpStr = record.getNBR_PRSULC_STG1();
                if (ValidateUtils.isValidValue(tmpStr, ValidateUtils.ARRAY_CARET_VALUES) || ValidateUtils.DOUBLE_ZERO.equals(tmpStr)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4170,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1324_STG_PRBLM_ULCER, problemUlcer)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4170,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1322_NBR_PRSULC_STG1, tmpStr)));
                }
            } else if ("02".equals(problemUlcer)) {
                // number of pressure Ulcer stage 2 must be > 0
                tmpStr = record.getNBR_PRSULC_STG2();
                if (ValidateUtils.isValidValue(tmpStr, ValidateUtils.ARRAY_CARET_VALUES) || ValidateUtils.DOUBLE_ZERO.equals(tmpStr)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4180,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1324_STG_PRBLM_ULCER, problemUlcer)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4180,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG2, tmpStr)));
                }
            } else if ("03".equals(problemUlcer)) {
                // number of pressure Ulcer stage 3 must be > 0
                tmpStr = record.getNBR_PRSULC_STG3();
                if (ValidateUtils.isValidValue(tmpStr, ValidateUtils.ARRAY_CARET_VALUES) || ValidateUtils.DOUBLE_ZERO.equals(tmpStr)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4190,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1324_STG_PRBLM_ULCER, problemUlcer)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4190,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG3, tmpStr)));
                }
            } else if ("04".equals(problemUlcer)) {
                // number of pressure Ulcer stage 4 must be > 0
                tmpStr = record.getNBR_PRSULC_STG4();
                if (ValidateUtils.isValidValue(tmpStr, ValidateUtils.ARRAY_CARET_VALUES) || ValidateUtils.DOUBLE_ZERO.equals(tmpStr)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4200,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1324_STG_PRBLM_ULCER, problemUlcer)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4200,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1308_NBR_PRSULC_STG4, tmpStr)));
                }
            }
        }

        return valid;
    }

    /**
     * Stasis Ulcers
     *
     * Only checked for Assessment Reason 01, 03, 04, 05
     *
     * @param record
     * @return
     */
    public boolean validateStasisUlcer(HomeHealthRecordIF record) {
        boolean valid = true;

        if (isRFA_1_3_4_5_9) {
            final String stasPresent = record.getSTAS_ULCR_PRSNT();
            final String numStas = record.getNBR_STAS_ULCR();
            final String statStas = record.getSTUS_PRBLM_STAS_ULCR();

            // 01 = patient has both observable and unobservable
            // 02 = patient has observable only
            if ("01".equals(stasPresent) || "02".equals(stasPresent)) {
                // if number of stasis ulcers or status of problem stasis ulcer are blank, then flag the error
                if (ValidateUtils.isValidValue(numStas, ValidateUtils.ARRAY_CARET_VALUES)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4230,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1332_NBR_STAS_ULCR, numStas)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4230,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1330_STAS_ULCR_PRSNT, stasPresent)));
                }

                if (ValidateUtils.isValidValue(statStas, ValidateUtils.ARRAY_CARET_VALUES)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4230,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1334_STUS_PRBLM_STAS_ULCR, statStas)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4230,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1330_STAS_ULCR_PRSNT, stasPresent)));
                }
            } else if ((ValidateUtils.DOUBLE_ZERO.equals(stasPresent) || "03".equals(stasPresent))) {
                // 00 = patient does not have any stasis ulcers
                // 03 = patient has only an unobservable stasis ulcer
                // if the stasis ulcer is 00 or 03, then the number of stasis
                // ulcers and status of problem stasis ulcer both should be blank
                if (!ValidateUtils.isValidValue(numStas, ValidateUtils.ARRAY_CARET_VALUES)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4220,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1332_NBR_STAS_ULCR, numStas)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4220,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1330_STAS_ULCR_PRSNT, stasPresent)));

                }
                if (!ValidateUtils.isValidValue(statStas, ValidateUtils.ARRAY_CARET_VALUES)) {
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4220,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1334_STUS_PRBLM_STAS_ULCR, statStas)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4220,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1330_STAS_ULCR_PRSNT, stasPresent)));
                    valid = false;
                }
            } else {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1330_STAS_ULCR_PRSNT, stasPresent)));
            }

            if (!ValidateUtils.isValidValue(numStas, ValidateUtils.ARRAY_DOUBLE_ONE_TWO_THREE_FOUR, ValidateUtils.ARRAY_CARET_VALUES)) {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1332_NBR_STAS_ULCR, numStas)));
            }

            if (!ValidateUtils.isValidValue(statStas, ValidateUtils.ARRAY_DOUBLE_ONE_TWO_THREE, ValidateUtils.ARRAY_CARET_VALUES)) {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1334_STUS_PRBLM_STAS_ULCR, statStas)));
            }
        }

        return valid;
    }

    /**
     * M1342_STUS_PRBLM_SRGCL_WND Status Of Most Problematic Surgical Wound
     *
     * Only checked for Assessment Reason 01, 03, 04, 05
     *
     * @param record
     * @return
     */
    public boolean validateSTATSURG(HomeHealthRecordIF record) {
        boolean valid = true;

        if (isRFA_1_3_4_5_9) {
            final String surgPresent = record.getSRGCL_WND_PRSNT();
            final String statSurg = record.getSTUS_PRBLM_SRGCL_WND();

            if (!ValidateUtils.isValidValue(statSurg, ValidateUtils.ARRAY_DOUBLE_ZERO_ONE_TWO_THREE, ValidateUtils.ARRAY_CARET_VALUES)) {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1342_STUS_PRBLM_SRGCL_WND, statSurg)));
            }

            if ("01".equals(surgPresent)) {
                // if there is an observable surgical wound, then
                // the status can not be blank/^
                if (ValidateUtils.isValidValue(statSurg, ValidateUtils.ARRAY_CARET_VALUES)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4250,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1340_SRGCL_WND_PRSNT, surgPresent)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4250,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1342_STUS_PRBLM_SRGCL_WND, surgPresent)));
                }

            } else if (ValidateUtils.DOUBLE_ZERO.equals(surgPresent) || "02".equals(surgPresent)) {
                if (!ValidateUtils.isValidValue(statSurg, ValidateUtils.ARRAY_CARET_VALUES)) {
                    // if there is no or unobservable surgical wound,
                    // then the status must be blank
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4240,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1340_SRGCL_WND_PRSNT, surgPresent)));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4240,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1342_STUS_PRBLM_SRGCL_WND, surgPresent)));
                }
            } else {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1340_SRGCL_WND_PRSNT, surgPresent)));
            }
        }

        return valid;
    }

    /**
     * M1400_WHEN_DYSPNEIC When Dyspneic
     *
     * Only checked for Assessment Reason 01, 03, 04, 05
     *
     * @param record
     * @return
     */
    public boolean validateWHEN_DYSPNEIC(HomeHealthRecordIF record) {
        boolean valid = true;

        if (isRFA_1_3_4_5_9
                && !ValidateUtils.isValidValue(record.getWHEN_DYSPNEIC(), ARRAY_DOUBLE_ZERO_ONE_TWO_THREE_FOUR)) {
            valid = false;
            this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                    new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1400_WHEN_DYSPNEIC, record.getWHEN_DYSPNEIC())));
        }
        return valid;
    }

    /**
     * M1610_UR_INCONT Urinary Incontinence or Urinary Catheter
     *
     * Only checked for Assessment Reason 01, 03, 04, 05
     *
     * @param record
     * @return
     */
    public boolean validateUR_INCONT(HomeHealthRecordIF record) {
        boolean valid = true;

        if (isRFA_1_3_4_5_9) {
            final String urIncontTiming = ((HomeHealthRecord_C1_IF) record).getINCNTNT_TIMING();
            final String urIncont = record.getUR_INCONT();

            // validate the range first
            if (!ValidateUtils.isValidValue(urIncont, ARRAY_DOUBLE_ZERO_ONE_TWO)) {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1610_UR_INCONT, urIncont)));
            }

            if (isRFA_1_3_9) {
                // validate the range first
                if (!ValidateUtils.isValidValue(urIncontTiming, ARRAY_DOUBLE_ZERO_ONE_TWO_THREE_FOUR, ValidateUtils.ARRAY_CARET_VALUES)) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1615_INCNTNT_TIMING, urIncontTiming)));
                }

                if (valid) {
                    if ((ValidateUtils.DOUBLE_ZERO.equals(urIncont) || "02".equals(urIncont))
                            && !ValidateUtils.isValidValue(urIncontTiming, ValidateUtils.ARRAY_CARET_VALUES)) {
                        valid = false;
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4330,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1615_INCNTNT_TIMING, urIncontTiming)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4330,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1610_UR_INCONT, urIncont)));

                    } else if (("01".equals(urIncont))
                            && ValidateUtils.isValidValue(urIncontTiming, ValidateUtils.ARRAY_CARET_VALUES)) {
                        valid = false;
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4340,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1615_INCNTNT_TIMING, urIncontTiming)));
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4340,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1610_UR_INCONT, urIncont)));

                    }
                }
            }
        }

        return valid;
    }

    /**
     * M1620_BWL_INCONT Bowel Incontinence Frequency
     *
     * Only checked for Assessment Reason 04, 05
     *
     * @param record
     * @return
     */
    public boolean validateBWL_INCONT(HomeHealthRecordIF record) {
        boolean valid = true;
        final String bowel = record.getBWL_INCONT();

        if (isRFA_1_3_4_5_9) {
            // if bowl incontenance is active, it must be one of the other valid values
            if (!ValidateUtils.isValidValue(bowel, ARRAY_DOUBLE_ZERO_ONE_TWO_THREE_FOUR_FIVE, ARRAY_NOT_APPLICABLE, ARRAY_UNKNOWN_UK)) {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1620_BWL_INCONT, bowel)));
            }

            // for assessment reasons 4, 5 or 9 bowel incontinence
            // can not be UK
            if (isRFA_4_5_9
                    && ValidateUtils.isValidValue(bowel, ARRAY_UNKNOWN_UK)) {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4350,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1620_BWL_INCONT, bowel)));
            }
        }

        return valid;
    }

    /**
     * M1630_OSTOMY Ostomy for Bowel Elimination
     *
     * Only checked for Assessment Reason 01, 03, 04, 05
     *
     * @param record
     * @return
     */
    public boolean validateBWL_INCONT_OSTOMY(HomeHealthRecordIF record) {
        boolean valid = true;

        if (isRFA_1_3_4_5) {
            final String bwlIncont = record.getBWL_INCONT();
            final String ostomy = record.getOSTOMY();

            if (ValidateUtils.isValidValue(ostomy, ARRAY_DOUBLE_ZERO_ONE_TWO)) {
                if (NOT_APPLICABLE.equals(bwlIncont)) {
                    if (!ValidateUtils.isValidValue(ostomy, ValidateUtils.ARRAY_DOUBLE_ONE_TWO)) {
                        // if bowl incontinence is NA, then the Ostomy
                        // must be 01 or 02
                        valid = false;
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4370,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1620_BWL_INCONT, bwlIncont)));
                    }
                } else if (ValidateUtils.isValidValue(bwlIncont, ARRAY_DOUBLE_ZERO_ONE_TWO_THREE_FOUR_FIVE, ARRAY_UNKNOWN_UK)
                        && !ValidateUtils.DOUBLE_ZERO.equals(ostomy)) {
                    // if the bowl incontinence is 00-05, or UK, then Ostomy
                    // must be 00
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4360,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1620_BWL_INCONT, bwlIncont)));
                }
            } else {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1630_OSTOMY, ostomy)));
            }
        }

        return valid;
    }

    /**
     * M2030_CRNT_MGMT_INJCTN_MDCTN Current: Management Of Injectable
     * Medications
     *
     * Only checked for Assessment Reason 01, 03, 04, and 05 - because the
     * related field Drug Regimen Review is only valid during 01 and 03, the
     * validation is split
     *
     * @param record
     * @return
     */
    public boolean validateCUR_INJECT_MEDS(HomeHealthRecordIF record) {
        boolean valid = true;

        if (isRFA_1_3_4_5_9) {
            final String tmpInjectMeds = record.getCRNT_MGMT_INJCTN_MDCTN();
            final String drugRegemine = ((HomeHealthRecord_C1_IF) record).getDRUG_RGMN_RVW();

            if (isRFA_1_3) {
                if (ValidateUtils.isValidValue(drugRegemine, ValidateUtils.ARRAY_DOUBLE_ZERO_ONE_TWO)) {
                    if (ValidateUtils.isValidValue(tmpInjectMeds, ValidateUtils.ARRAY_CARET_VALUES)) {
                        valid = false;
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4450,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M2030_CRNT_MGMT_INJCTN_MDCTN, tmpInjectMeds)));
                    }
                } else if (ValidateUtils.NOT_APPLICABLE.equals(drugRegemine)) {
                    if (!ValidateUtils.isValidValue(tmpInjectMeds, ValidateUtils.ARRAY_CARET_VALUES)) {
                        valid = false;
                        this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4440,
                                new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M2030_CRNT_MGMT_INJCTN_MDCTN, tmpInjectMeds)));
                    }
                } else {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M2000_DRUG_RGMN_RVW, drugRegemine)));
                }
            }

            if (isRFA_4_5_9
                    && ValidateUtils.isValidValue(tmpInjectMeds, ValidateUtils.ARRAY_CARET_VALUES)) {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4490,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M2030_CRNT_MGMT_INJCTN_MDCTN, tmpInjectMeds)));
            }

            if (!ValidateUtils.isValidValue(tmpInjectMeds, ValidateUtils.ARRAY_DOUBLE_ZERO_ONE_TWO_THREE, ValidateUtils.ARRAY_NOT_APPLICABLE, ValidateUtils.ARRAY_CARET_VALUES)) {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M2030_CRNT_MGMT_INJCTN_MDCTN, tmpInjectMeds)));
            }
        }

        return valid;
    }
    /*------------------------------------------------------------
     * FUNCTIONAL DOMAIN VALIDATIONS
     * -------------------------------------------------------- */

    /**
     * M1810_CUR_DRESS_UPPER Current: Dress Upper Body and M1820_CUR_DRESS_LOWER
     * Current: Dress Lower Body
     *
     * Only checked for Assessment Reason 01, 03, 04, 05
     *
     * @param record
     * @return
     */
    public boolean validateCUR_DRESS(HomeHealthRecordIF record) {
        boolean valid = true;

        if (isRFA_1_3_4_5_9) {
            if (!ValidateUtils.isValidValue(record.getCRNT_DRESS_UPPER(), ARRAY_DOUBLE_ZERO_ONE_TWO_THREE)) {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1810_CRNT_DRESS_UPPER, record.getCRNT_DRESS_UPPER())));
            }
            if (!ValidateUtils.isValidValue(record.getCRNT_DRESS_LOWER(), ARRAY_DOUBLE_ZERO_ONE_TWO_THREE)) {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1820_CRNT_DRESS_LOWER, record.getCRNT_DRESS_LOWER())));
            }
        }

        return valid;
    }

    /**
     * M1830_CRNT_BATHG Current: Bathing
     *
     * Only checked for Assessment Reason 01, 03, 04, 05
     *
     * @param record
     * @return
     */
    public boolean validateCUR_BATHING(HomeHealthRecordIF record) {
        boolean valid = true;

        if (isRFA_1_3_4_5_9
                && !ValidateUtils.isValidValue(record.getCRNT_BATHG(), ARRAY_DOUBLE_ZERO_ONE_TWO_THREE_FOUR_FIVE_SIX)) {
            valid = false;
            this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                    new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1830_CRNT_BATHG, record.getCRNT_BATHG())));
        }

        return valid;
    }

    /**
     * M1840_CUR_TOILTG Current: Toileting
     *
     * Only checked for Assessment Reason 01, 03, 04, 05
     *
     * @param record
     * @return
     */
    public boolean validateCUR_TOILETING(HomeHealthRecordIF record) {
        boolean valid = true;
        
        if (isRFA_1_3_4_5_9
                && !ValidateUtils.isValidValue(record.getCRNT_TOILTG(), ARRAY_DOUBLE_ZERO_ONE_TWO_THREE_FOUR)) {
            valid = false;
            this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                    new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1840_CRNT_TOILTG, record.getCRNT_TOILTG())));
        }

        return valid;
    }

    /**
     * M1850_CUR_TRNSFRNG Current: Transferring
     *
     * Only checked for Assessment Reason 01, 03, 04, 05
     *
     * @param record
     * @return
     */
    public boolean validateCUR_TRANSFERRING(HomeHealthRecordIF record) {
        boolean valid = true;

        if (isRFA_1_3_4_5_9
                && !ValidateUtils.isValidValue(record.getCRNT_TRNSFRNG(), ARRAY_DOUBLE_ZERO_ONE_TWO_THREE_FOUR_FIVE)) {
            valid = false;
            this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                    new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1850_CRNT_TRNSFRNG, record.getCRNT_TRNSFRNG())));
        }

        return valid;
    }

    /**
     * M1860_CRNT_AMBLTN Current: Ambulation
     *
     * Only checked for Assessment Reason 01, 03, 04, 05
     *
     * @param record
     * @return
     */
    public boolean validateCUR_AMBULATION(HomeHealthRecordIF record) {
        boolean valid = true;

        if (isRFA_1_3_4_5_9
                && !ValidateUtils.isValidValue(record.getCRNT_AMBLTN(), ARRAY_DOUBLE_ZERO_ONE_TWO_THREE_FOUR_FIVE_SIX)) {
            valid = false;
            this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                    new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M1860_CRNT_AMBLTN, record.getCRNT_AMBLTN())));
        }

        return valid;
    }

    /*------------------------------------------------------------
     * SERVICE DOMAIN VALIDATIONS
     * -------------------------------------------------------- */
    /**
     * M2200_THER_NEED_NBR Therapy Need: Number of Visits
     *
     * M2200_THER_NEED_NA Therapy Need: Not Applicable
     *
     * Only checked for Assessment Reason 01, 03, 04, 05
     *
     * @param record
     * @return
     */
    public boolean validateTHER_NEED(HomeHealthRecordIF record) {
        boolean valid = true;

        if (isRFA_1_3_4_5) {
            final String tmpStr = record.getTHER_NEED_NA();
            final int needNumber = record.getTHER_NEED_NBR();

            if ("1".equals(tmpStr)) {
                /*
                 * M0826_THER_NEED_NBR is an int, so an invalid or
                 * blank value would be -1, and entered number would
                 * be >= 0
                 */
                if (needNumber >= 0) {
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4520,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M2200_THER_NEED_NBR, Integer.toString(record.getTHER_NEED_NBR()))));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4520,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M2200_THER_NEED_NA, tmpStr)));
                }
            } else if ("0".equals(tmpStr)) {
                if (needNumber < 0 || needNumber > 999) {
                    // ensure the therapy count is in the valid range
                    valid = false;
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4510,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M2200_THER_NEED_NBR, Integer.toString(record.getTHER_NEED_NBR()))));
                    this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_4510,
                            new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M2200_THER_NEED_NA, tmpStr)));
                }
            } else {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M2200_THER_NEED_NA, tmpStr)));
            }

            /*
             * negative 1 is the value of empty, anything less is an error value
             */
            if (needNumber < -1) {
                valid = false;
                this.edits.add(new OasisValidationEdit(OasisEditsEN.EDIT_3060,
                        new HHOasisDataItem(HomeHealthRecord_C1_IF.OASIS_C1_ITEM_M2200_THER_NEED_NBR, Integer.toString(needNumber))));
            }
        }
        return valid;
    }
}
